import React, { useEffect, useState } from 'react';
import styled from 'styled-components';
import axios from 'axios';
import moment from 'moment';

import { Button } from 'react-bootstrap';
import { RefreshCw } from 'react-feather';
import Markdown from 'react-markdown';
import gfm from 'remark-gfm';

import { ModalContextProvider } from '@common/providers/modal/provider';
import { useModal } from '@common/hooks/useModal';

import JSStatistics from '../../components/Block/Statistics';
import StatisticsBlock, {
  JSStatsBlock,
  StatsContent,
  //--------------
  StatisticHeader,
  StatisticItemSpacedContent,
  StatisticLabel,
} from '../../components/Block/StatsBlock';

import { BlockTabHeader } from './BlockTab';
import { useShowKpiModal, KpiModalCall } from './useShowKpiModal';
import { KpiModals } from './KpiModals';

import { baseUrl } from '../constants';
// import TOC from './TOC';
// import Stats from './Stats';
// import KPI from './KpiBox';

import {
  List,
  ListColumn,
  ListText,
  ListRow,
  ListLabel,
} from '../../components/List';

import { MarkdownText } from '../utils';

interface Kpi {
  active: string;
  distribution: string;
  fiatPool: string;
  process: string;
  purpose: string;
  reward: number;
  rewardDistribution: string;
  scope: string[];
  title: string;
}

interface KpiSection {
  id: string;
  kpis: Kpi;
}

interface Round {
  cms: number;
  councilMembers: number;
  deadline: string;
  deadlineBlock: number;
  electionRound: number;
  endBlock: number;
  endDate: string;
  fiatDiff: string;
  kpis: {
    sections: KpiSection[];
  };
}

const KpiLabel = styled(StatisticLabel)`
  cursor: pointer;
  &:hover {
    text-decoration: underline;
  }
`;

function BigTabBlockItem({ children, data, setShowModal }: any) {
  const showKpiModal = useShowKpiModal(data); // TODO

  // console.log('showKpiModal', showKpiModal);

  return <KpiLabel onClick={showKpiModal}>{children}</KpiLabel>;
}

export function BigTabBlock({
  title,
  items,
  round,
  section,
}: {
  round: Round;
}) {
  const [showModal, setShowModal] = useState(null);
  const useModalData = useModal<KpiModalCall>();

  // console.log('modalData = ', useModalData);

  // const handlerOpen = (modalData) => setShowModal(show)

  return (
    <JSStatsBlock size="s">
      <BlockTabHeader title={title} />
      <StatsContent>
        {items &&
          items.map((item, i: number) => (
            <StatisticItemSpacedContent>
              <BigTabBlockItem data={item} setShowModal={setShowModal}>
                {`${round}-${section.id}-${i + 1}`} {item.title}
              </BigTabBlockItem>
            </StatisticItemSpacedContent>
          ))}
      </StatsContent>
    </JSStatsBlock>
  );
}

const Stats = ({ round }) => {
  const {
    electionRound,
    councilMembers,
    fiatDiff,
    reward,
    startBlock,
    startDate,
    endBlock,
    endDate,
    deadline,
    deadlineBlock,
    notes,
  } = round;

  const length = moment(endDate, 'DD.MM.YY').diff(
    moment(startDate, 'DD.MM.YY'),
    'days'
  );

  return (
    <>
      <List>
        <ListRow>
          <ListLabel text="Total Possible Rewards" />
          <ListText>${reward}</ListText>
        </ListRow>
        <ListRow>
          <ListLabel text="Max Fiat Pool Difference" />
          <ListText>${fiatDiff}</ListText>
        </ListRow>
        <ListRow>
          <ListLabel text="Council Elected in Round" />
          <ListText>{electionRound}</ListText>
        </ListRow>
        <ListRow>
          <ListLabel text="Council Members" />
          <ListText>{councilMembers}</ListText>
        </ListRow>
        <ListRow>
          <ListLabel text="KPI Length" />
          <ListText>
            ${endBlock - startBlock} ({length} days)
          </ListText>
        </ListRow>
        <ListRow>
          <ListLabel text="Start Block/Date" />
          <ListText>
            #{startBlock} ({startDate})
          </ListText>
        </ListRow>
        <ListRow>
          <ListLabel text="End Block/Date" />
          <ListText>
            #{endBlock} ({endDate})
          </ListText>
        </ListRow>
        <ListRow>
          <ListLabel text="Deadline to Submit Summary" />
          <ListText>
            {deadlineBlock} ({deadline})
          </ListText>
        </ListRow>
      </List>
      <MarkdownText>
        <Markdown
          plugins={[gfm]}
          className="overflow-auto text-left"
          children={notes}
        />
      </MarkdownText>
    </>
  );
};

const Round = (props: any) => {
  const [roundData, setRound] = useState<Round>({});

  // console.log('Round', roundData);

  const fetchData = () => {
    axios
      .get(`${baseUrl}/kpi.json`)
      .then(({ data }) => {
        const rounds = data.rounds;
        console.debug(`Received KPI for ${rounds.length} round(s).`);
        const round = rounds.reduce(
          (r, max) => (r.round > max.round ? r : max),
          rounds[0]
        );

        setRound({ rounds, round });
      })
      .catch((e) => console.error(`Failed to fetch KPI data.`, e));
  };

  useEffect(() => {
    fetchData();
  }, []);

  console.log('roundData', roundData);

  // const { toggleEditKpi, focus, fetchKpi } = props;
  const { round /* notes, startBlock, endBlock */ } = roundData;

  if (true) {
    return (
      <>
        <ModalContextProvider>
          <JSStatistics>
            {round?.kpis?.sections.map((section, index: number) => (
              <>
                <BigTabBlock
                  title={section.name}
                  items={section.kpis}
                  section={section}
                  round={round.round}
                />
                {/* <div key={index} id={`${round}-${section.id}`}>
                <h4>{section.name}</h4>
                <div className="d-flex flex-wrap">
                  {section.kpis.map((kpi, i: number) => (
                    <KPI
                      key={i}
                      id={`${round}-${section.id}-${i + 1}`}
                      toggleEdit={toggleEditKpi}
                      kpi={kpi}
                      start={startBlock}
                      end={endBlock}
                    />
                  ))}
                </div>
              </div> */}
              </>
            ))}
          </JSStatistics>
          {round && <Stats round={round} />}
          <KpiModals />
        </ModalContextProvider>
      </>
    );
  }

  /*  return (
    <div>
      <Button variant="" className="p-1 btn-sm float-right">
        <RefreshCw onClick={fetchKpi} />
      </Button>

      <div className="d-flex flex-wrap">
        <div className="col-12 col-lg-5">
          <TOC focus={focus} round={props.round} />
          <h3>Round {round}</h3>
          <Stats round={props.round} />

          <Markdown
            plugins={[gfm]}
            className="overflow-auto text-left"
            children={notes}
          />
        </div>
        <div className="col-12 col-lg-7">
          {kpis.sections.map((section, index: number) => (
            <div key={index} id={`${round}-${section.id}`}>
              <h4>{section.name}</h4>
              <div className="d-flex flex-wrap">
                {section.kpis.map((kpi, i: number) => (
                  <KPI
                    key={i}
                    id={`${round}-${section.id}-${i + 1}`}
                    toggleEdit={toggleEditKpi}
                    kpi={kpi}
                    start={startBlock}
                    end={endBlock}
                  />
                ))}
              </div>
            </div>
          ))}
        </div>
      </div>
    </div>
  ); */
};

export default React.memo(Round);
